--[[
Name: LibSpecialEvents-Skill-4.0
Revision: $Rev: 78039 $
Author: LordFarlander
Website: http://www.wowace.com/
Description: Special events for skills (learned, leveled, unlearned)
Dependencies: LibStub, CallbackHandler-1.0
]]

local vmajor, vminor = "LibSpecialEvents-Skill-4.0", "$Revision: 78039 $";

local lib, oldMinor = LibStub:NewLibrary( vmajor, vminor );
if( not lib ) then
    return;
end

local frame;
if lib.frame then
    frame = lib.frame;
    frame:UnregisterAllEvents();
    frame:SetScript( "OnEvent", nil );
else
    frame = CreateFrame( "Frame", vmajor .. "_Frame" );
    lib.frame = frame;
end

frame:RegisterEvent( "CHAT_MSG_SKILL" );

frame:SetScript("OnEvent",
    function( this, event, ... )
        this[event]( lib, ... );
    end );

local CallbackHandler = LibStub:GetLibrary( "CallbackHandler-1.0" );
if lib.callbacks then
    lib:UnregisterAll( lib );
    lib:UnhookDialogs();
else
    lib.callbacks = LibStub( "CallbackHandler-1.0" ):New( lib, nil, nil, "UnregisterAll" );
end

if( not lib.vars ) then
    lib.vars = {};
end

--Create versions of the chat messages that we can use to scan
SPECIALEVENTS_ERR_SKILL_GAINED_S = string.gsub( ERR_SKILL_GAINED_S, "%%s", "(.+)" );
SPECIALEVENTS_ERR_SKILL_UP_SI = string.gsub( ERR_SKILL_UP_SI, "%%s", "(.+)" );
SPECIALEVENTS_ERR_SKILL_UP_SI = string.gsub( SPECIALEVENTS_ERR_SKILL_UP_SI, "%%1$s", "(.+)" );
SPECIALEVENTS_ERR_SKILL_UP_SI = string.gsub( SPECIALEVENTS_ERR_SKILL_UP_SI, "%%d", "(%%d+)" );
SPECIALEVENTS_ERR_SKILL_UP_SI = string.gsub( SPECIALEVENTS_ERR_SKILL_UP_SI, "%%2$d", "(%%d+)" );
SPECIALEVENTS_ERR_SPELL_UNLEARNED_S = string.gsub( ERR_SPELL_UNLEARNED_S, "%%s", "(.+)" );

--------------------------------
--      Tracking methods      --
--------------------------------

function frame:CHAT_MSG_SKILL( message )
    --Figure out WHICH chat message was just fired and get some useful info about it
    local skillName, level;

    if( not lib.vars.skills ) then
        lib.vars.skills = lib:GetSkillList();
    end
    skillName = string.match( message, SPECIALEVENTS_ERR_SKILL_GAINED_S );
    if( skillName ) then
        lib.vars.skills[skillName] = 1;
        lib.callbacks:Fire( "LearnedSkill", skillName );
    else
        skillName = string.match( message, ERR_SPELL_UNLEARNED_S );

        if( skillName and lib.skills[skillName] ) then
            lib.vars.skills[skillName] = nil;
            lib.callbacks:Fire( "UnlearnedSkill", skillName );
        else
            skillName, level = string.match( message, SPECIALEVENTS_ERR_SKILL_UP_SI );

            if( skillName and level ) then
                level = tonumber( level );
                lib.vars.skills[skillName] = level;
                lib.callbacks:Fire( "SkillLeveled", skillName, level );
            end--if
        end--if
    end--if
end

function lib:GetSkillList()
    local expandedHeaders = {};
    local skills = {};
    local returnValue = 0;

    for i = 0, GetNumSkillLines(), 1 do
        local skillName, header, isExpanded, _ = GetSkillLineInfo( i );

        if ( header and not isExpanded ) then
            expandedHeaders[i] = skillName;
        end--if
    end--for
    ExpandSkillHeader( 0 );
    for i = 1, GetNumSkillLines(), 1 do
        local skillName, header, _, skillRank = GetSkillLineInfo( i );

        if( skillName and ( not header ) ) then
            skills[skillName] = skillRank;
        end--if
    end--for
    for i = 0, GetNumSkillLines() do
        local skillName, header, isExpanded = GetSkillLineInfo( i );

        for j, name in pairs( expandedHeaders ) do
            if( header and ( skillName == name ) ) then
                CollapseSkillHeader( i );
                expandedHeaders[j] = nil;
            end--if
        end--for
    end--for
    return skills;
end

-----------------------------
--      Query methods      --
-----------------------------

function lib:GetSkills()
    if( not self.vars.skills ) then
        self.vars.skills = self:GetSkillList();
    end
    return self.vars.skills;
end

function lib:GetSkillLevel( skill )
    if( not self.vars.skills ) then
        self.vars.skills = self:GetSkillList();
    end
    if( self.vars.skills[skill] ) then
        return self.vars.skills[skill];
    end--if
    return 0;
end
